/* $Id: sfskeygen.C,v 1.15 1999/04/22 03:00:45 kaminsky Exp $ */

/*
 *
 * Copyright (C) 1999 Michael Kaminsky (kaminsky@lcs.mit.edu)
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "sfskey.h"

void
keygen::finish (int err)
{
  (*cb) (err);
  delete this;
}

void
keygen::decnevents ()
{
  if (!--nevents)
    savekey ();
}

void
keygen::savekey ()
{
  if (!key) {
    warnx << "Please wait...";
    err_flush ();
    key = New refcounted<rabin_priv> (rabin_keygen (bits));
    warnx << "\n";
  }

  str s = export_rabin_priv (*key, pwd, comment);

  ptr<rabin_priv> x = import_rabin_priv (s, pwd);
  assert (x);
  assert (key->p == x->p);
  assert (key->q == x->q);

  if (!str2file (keyfile, s, 0600))
    fatal << keyfile << ": " << strerror (errno) << "\n";

  finish (0);
}

void
keygen::callkbdnoise ()
{
  if (opt_kbdnoise) {
    warnx << "Please type some random text with which to seed the "
      "pseudo random generator:\n";
    if (!getkbdnoise (64, &rnd_input, wrap (this, &keygen::decnevents)))
      fatal << "stdin must be a tty with -k option\n";
  }
  else
    decnevents ();
}

void
keygen::gotpwd (bool confirm, str s)
{
  if (!confirm) {
    if (s == "") {
      warnx ("Aborted.\n");
      finish (1);
    }
    pwd = s;
    getkbdpwd ("          Again: ", &rnd_input, 
	       wrap (this, &keygen::gotpwd, true));
  }
  else if (s != pwd) {
    warnx << "Mismatch; try again, or RETURN to quit.\n";
    getkbdpwd ("      Passprase: ", &rnd_input, 
	       wrap (this, &keygen::gotpwd, false));
  }
  else
    callkbdnoise ();
}

void
keygen::newkey ()
{
  warnx << "Creating new key for " << keyfile << ".\n";

  nevents = 2;
  if (opt_nopwd)
    callkbdnoise ();
  else if (!getkbdpwd ("Enter passprase: ", &rnd_input, 
		       wrap (this, &keygen::gotpwd, false)))
    fatal ("stdin must be a tty (or use -P for no passphrase)\n");
  getsysnoise (&rnd_input, wrap (this, &keygen::decnevents));
}

void
keygen::reencrypt (ptr<rabin_priv> skp)
{
  if (!skp)
    fatal << "Could not decode " << keyfile << ".\n";

  key = skp;
  if (opt_nopwd)
    callkbdnoise ();
  else if (!getkbdpwd ("  New passphrase: ", &rnd_input, 
		       wrap (this, &keygen::gotpwd, false)))
    fatal ("stdin must be a tty (or use -P for no passphrase)\n");
}

void
keygen::editkey (str asc)
{
  if (opt_b)
    warnx << "Warning: -b flag ignored for existing key\n";
  if (opt_nopwd)
    warnx << "Removing passphrase from " << keyfile << ".\n";
  else
    warnx << "Changing passphrase for " << keyfile << ".\n";

  if (opt_nopwd && !import_rabin_priv_need_pwd (asc)) {
    if (!import_rabin_priv (asc, NULL))
      fatal << "Could not decode " << keyfile << ".\n";
    warnx << keyfile << " already had no passphrase.\n";
    finish (0);
    return;
  }

  nevents = 2;
  if (!import_rabin_priv_askpwd (asc, "  Old passphrase: ", 
				 wrap (this, &keygen::reencrypt),
				 !comment ? &comment : NULL))
    fatal ("stdin must be a tty to read old passphrase.\n");
  getsysnoise (&rnd_input, wrap (this, &keygen::decnevents));
}

keygen::keygen (str kf, str cmt, int bits, bool nopwd, bool kbdnoise, 
		bool b, cb_t cb)
  : comment (cmt), keyfile (kf), bits (bits), opt_nopwd (nopwd), 
    opt_kbdnoise (kbdnoise), opt_b (b), cb (cb)
{
  /* XXX:  Eventually we may want to put the random_seed elsewhere */
  usrdir_make ();
  str rseedfile = dotsfs << "/random_seed";
  random_set_seedfile (rseedfile);

  str oldkey = file2wstr (keyfile);
  if (!oldkey) {
    if (errno == ENOENT)
      newkey ();
    else
      fatal << keyfile << ": " << strerror (errno) << "\n";
  }
  else
    editkey (oldkey);
}
