// -*-c++-*-
/* $Id: srp.h,v 1.8 1999/04/14 19:51:55 kaminsky Exp $ */

/*
 *
 * Copyright (C) 1999 Michael Kaminsky (kaminsky@lcs.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

/*
 * Secure Remote Password Protocol
 *   http://srp.stanford.edu/srp/
 *
 */

#ifndef _SRP_H_
#define _SRP_H_ 1

#include "bigint.h"
#include "sha1.h"
#include "blowfish.h"

class srp {
  enum { UBITS = 1024 };
  typedef u_char hash_t[20];
  void pickN (u_int nbits);
  void pickg ();
  void calc_K_cproof ();
  void calc_sproof ();

public:
  bigint N, g;
  bigint A, a;
  bigint B, b;
  bigint S;
  bigint v;
  bigint u;
  hash_t proof;
  hash_t K;
  str salt;
  str username;

  srp () : N (0), g (0) {}

  bool set_params (bigint Narg, bigint garg, bool check = true);
  void gen_params (u_int nbits) { pickN (nbits); pickg (); }
  bigint calc_verifier (bigint x) { return powm (g, x, N); }
  bool phase0serv (bigint Varg, str _salt, str _user);
  bool phase1clnt ();
  bool phase1serv (bigint Aarg);
  bool phase2clnt (bigint xArg, bigint Barg, bigint uarg, 
		   str _salt, str _user);
  bool phase2serv (hash_t clntproof);
  bool phase3clnt (hash_t servproof);
};

ptr<srp> import_srp_params (str raw);
str export_srp_params (const srp &s);

#endif /* !_SRP_H_  */
