#define USE_PCTR 0

#include "crypt.h"
#include "esign.h"
#include "rabin.h"
#include "bench.h"
#include "tiger.h"
#include "modalg.h"

class revdump {
  friend const strbuf &strbuf_cat (const strbuf &, const revdump &);
  const void *const buf;
  const size_t len;
public:
  revdump (const void *b, size_t l) : buf (b), len (l) {}
};

inline const strbuf &
strbuf_cat (const strbuf &sb, const revdump &hd)
{
  const u_char *s = static_cast<const u_char *> (hd.buf);
  const u_char *e = s + hd.len;
  while (e > s)
    sb.fmt ("%02x", *--e);
  return sb;
}


void
tigerfile (const char *file)
{
  int fd = open (file, O_RDONLY);
  if (fd < 0)
    return;

  int n;
  char buf[8192];
  tigerctx tc;

  while ((n = read (fd, buf, sizeof (buf))) > 0)
    tc.update (buf, n);

  u_char res[tigerctx::hashsize];
  tc.final (res);


  warn << file << ": " << revdump (res, sizeof (res)) << "\n";
  close (fd);
}

void
xxx_square (MP_INT *res, const MP_INT *arg)
{
  MP_INT tmp, *r;

  if (!arg->_mp_size) {
    res->_mp_size = 0;
    return;
  }
  if (res == arg) {
    r = &tmp;
    mpz_init (r);
  }
  else
    r = res;

  int asize = ABS (arg->_mp_size);
  const mp_limb_t *ap = arg->_mp_d;

  int rsize = 2 * asize;
  if (r->_mp_alloc < rsize)
    _mpz_realloc (r, rsize);
  mp_limb_t *rp = r->_mp_d;

  mpn_mul_n (rp, ap, ap, asize);

  while (rsize && !rp[rsize - 1])
    rsize--;
  r->_mp_size = rsize;
  if (res == arg) {
    mpz_clear (res);
    *(MP_INT *) res = tmp;
  }
}

int
main (int argc, char **argv)
{
  // random_update ();

  bigint m, m2, r, r2, ri, s1, s2;
  montgom b;

#if 1
  for (int i = 120; i < 162; i++) {
    //for (int i = 160; i >= 120; i--) {
    //for (int i = 500; i < 550; i++) {
    int res = 0;
    m = random_bigint (i);
    m.setbit (0, 1);
    b.set (m);
    m2 = m * b.getr ();
    for (int j = i - 33; j <= 2 * i; j++) {
      r = random_zn (m2);
      r.trunc (j);
      s1 = mod (r * b.getri (), m);
      s2 = b.mreduce (r);
      if (s1 != s2) {
	res |= 1;
#if 1
	panic << "mreduce failed\n"
	      << " m = " << m << "\n"
	      << " r = " << r << "\n"
	      << "     " << s1 << "\n  != " << s2 << "\n"
	      << "    ["
	      << strbuf ("%*s", mpz_sizeinbase (&s1, 16),
			 bigint (abs (s1 - s2)).cstr ())
	      << "]\n";
#endif
      }
    }

    // r = s1;
    r = random_zn (m);
    r2 = random_zn (m);
    assert (r < m && r2 < m);

    s1 = mod (r * r2 * b.getri (), m);
    b.mpz_mmul (&s2, &r, &r2);
    if (s1 != s2) {
      res |= 2;
#if 1
      panic << "mmul failed\n"
	    << " m = " << m << "\n"
	    << " r = " << r << "\n"
	    << "     " << s1 << "\n  != " << s2 << "\n"
	    << "    ["
	    << strbuf ("%*s", mpz_sizeinbase (&s1, 16),
		       bigint (abs (s1 - s2)).cstr ())
	    << "]\n";
#endif
    }

    s1 = powm (r, r2, m);
    b.mpz_powm (&s2, &r, &r2);
    if (s1 != s2) {
      res |= 4;
#if 0
      panic << "powm failed\n"
	    << " m = " << m << "\n"
	    << " r = " << r << "\n"
	    << "     " << s1 << "\n  != " << s2 << "\n"
	    << "    ["
	    << strbuf ("%*s", mpz_sizeinbase (&s1, 16),
		       bigint (abs (s1 - s2)).cstr ())
	    << "]\n";
#endif
    }

    warn ("%s mreduce.. %d\n", (res&1) ? "fail" : "ok", i);
    warn ("%s mmul.. %d\n", (res&2) ? "fail" : "ok", i);
    warn ("%s powm.. %d\n", (res&4) ? "fail" : "ok", i);
  }
#endif

  m = random_bigint (1024);
  r = random_bigint (2048);
  m.setbit (0, 1);
  b.set (m);

  BENCH (10240, s2 = b.mreduce (r));
  BENCH (10240, s1 = s2 * s2);
  BENCH (10240, b.mpz_mmul (&s1, &s2, &s2));

#if 0
  random_update ();
  // esign_priv key = esign_keygen (1536);
  // esign_priv key = esign_keygen (1024);
  rabin_priv key = rabin_keygen (1024);

  mstr msgbuf (32);
  rnd.getbytes (msgbuf, 32);
  str msg = msgbuf;

  bigint sig = key.sign (msg);

  BENCH (128, sig = key.sign (msg));
  BENCH (128, key.verify (msg, sig));
#endif

#if 0
  for (int i = 1; i < argc; i++)
    tigerfile (argv[i]);
#endif

  return 0;
}
