/* $Id: afs.C,v 1.26 1999/03/25 06:05:42 dm Exp $ */

/*
 *
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "sfscd.h"

bool terminating;

ref<afsroot> afs_root = afsroot::alloc ();
ref<afsdir> afs_sfsroot = afsusrdir::alloc (afs_root);
ref<afsdir> afs_naroot = afsusrdir::alloc (afs_root);

ref<afsdir> afs_mnt = afs_root->mkdir (".mnt");
ref<afsdir> afs_wait = afsdir::alloc ();

int afsfd;
ptr<asrv> afssrv;
ptr<asrv> afssrv3;

class idlink : public afslink {
  u_int32_t fhctr;
public:
  idlink () : fhctr (0) {}

  void mkfh (nfs_fh *fhp) {
    afsnode::mkfh (fhp);
    ((u_int32_t *) fhp)[2] = ++fhctr;
  }
  bool chkfh (const nfs_fh *) { return true; }

  void nfs_readlink (svccb *sbp) {
    if (const authunix_parms *aup = sbp->getaup ())
      setres (strbuf ("%d", aup->aup_uid));
    else
      setres (NFSERR_ACCES);
    sendreply (sbp);
  }
};

static void
afs_dispatch (svccb *sbp)
{
  if (sbp->vers () == 3)
    afsnode::dispatch3 (sbp);
  else
    afsnode::dispatch (sbp);
}

void afs_shutdown (bool);
void
afs_shutdown_cb (int stat = 0)
{
  if (!stat)
    exit (0);
  warn ("unmountall: %s\n", strerror (stat));
  timecb (time (NULL) + 5, wrap (afs_shutdown, false));
}
void
afs_shutdown (bool start)
{
  static int started;
  terminating = true;
  if (start) {
    if (started)
      return;
    started = 1;
    warn << "received signal, shutting down\n";
  }
  mnt_umountall (NUOPT_FORCE, wrap (afs_shutdown_cb));
}

static void
exit1 (int)
{
  exit (1);
}
static void afs_init2 (cbv, int, u_int64_t);
static void afs_init3 (cbv, int, u_int64_t);
void
afs_init (cbv cb)
{
  tzset ();

  rnd.getbytes (fhsecret, sizeof (fhsecret));

#if 0
  // XXX
  ptr<idlink> id = New refcounted<idlink>;
  afs_root->link (id, ".id0123456789abcdef0123456789abcdef");
  id = New refcounted<idlink>;
  afs_root->link (id, ".id");
#endif

  afs_mnt->mkdir ("wait");
  afs_sfsroot->mkdir (".root");

  afsfd = inetsocket (SOCK_DGRAM, nomounting ? 2490 : 0);
  if (afsfd < 0)
    fatal ("afs_init: inetsocket: %m\n");
  afssrv = asrv::alloc (axprt_dgram::alloc (afsfd),
			nfs_program_2, wrap (afs_dispatch));
  afssrv3 = asrv::alloc (axprt_dgram::alloc (afsfd),
			 nfs_program_3, wrap (afs_dispatch));

  sigcb (SIGINT, wrap (afs_shutdown, true));
  sigcb (SIGTERM, wrap (afs_shutdown, true));

  nfs_fh fh;
  afs_root->mkfh (&fh);
  mnt_mount (dup (afsfd), "(sfs)", sfsroot,
	     V3FLAG | NMOPT_NOAC | NMOPT_RO | NMOPT_SOFT,
	     nfs_fh2tobytes (fh), wrap (afs_init2, cb));
}
static void
afs_init2 (cbv cb, int stat, u_int64_t)
{
  if (stat)
    fatal ("mount (%s): %s\n", sfsroot, strerror (stat));

  nfs_fh fh;
  afs_wait->mkfh (&fh);
  mnt_mount (dup (afsfd), "(sfswait)", strbuf ("%s/.mnt/wait", sfsroot),
	     V3FLAG | NMOPT_NOAC | NMOPT_RO | NMOPT_SOFT,
	     nfs_fh2tobytes (fh), wrap (afs_init3, cb));
}
static void
afs_init3 (cbv cb, int stat, u_int64_t)
{
  if (stat) {
    warn << "fatal: mount (" << sfsroot << "/.mnt/wait" << "): "
	 << strerror (stat) << "\n";
    mnt_umountall (NUOPT_FORCE, wrap (exit1));
  }
  (*cb) ();
}
